/*
 * @Author: zuley
 * @Date: 2021-06-01 17:28:44
 * @LastEditors: zuley
 * @LastEditTime: 2021-06-04 18:46:02
 */

interface EnvPipeOpt<T> {
  /** 环境名称 */
  name: string;
  /** 环境判断函数 */
  in: () => boolean;
  /**
   * 匹配成功的触发函数
   */
  complete: () => Promise<T>;
}

export default class EnvPipe<T> {
  // 环境处理管道列表
  private envList: {
    [prop: string]: EnvPipeOpt<T>;
  } = {}
  // 默认函数
  public defaultPipe (): Promise<T> {
    console.log('没有设置默认管道方法')
    return null as any
  }
  // 推入函数
  public push (env: EnvPipeOpt<T>) {
    if (!this.envList[env.name]) {
      this.envList[env.name] = env
    } else {
      console.log("环境重复，请检查 PUSH 的参数")
    }
  }
  // 启动函数
  public start (config: string[]): Promise<T> {
    return new Promise((resolve) => {
      let actEnv
      for (const envName of config) {
        const env = this.envList[envName]
        if (env && env.in()) {
          actEnv = env
          resolve(env.complete())
          break
        }
      }
      if (!actEnv) resolve(this.defaultPipe())
    })
  }
}